# --- ROCK-COPYRIGHT-NOTE-BEGIN ---
# 
# This copyright note is auto-generated by ./scripts/Create-CopyPatch.
# Please add additional copyright information _after_ the line containing
# the ROCK-COPYRIGHT-NOTE-END tag. Otherwise it might get removed by
# the ./scripts/Create-CopyPatch script. Do not edit this copyright text!
# 
# ROCK Linux: rock-src/package/ia32/grub/stone_mod_grub.sh
# ROCK Linux is Copyright (C) 1998 - 2003 Clifford Wolf
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. A copy of the GNU General Public
# License can be found at Documentation/COPYING.
# 
# Many people helped and are helping developing ROCK Linux. Please
# have a look at http://www.rocklinux.org/ and the Documentation/TEAM
# file for details.
# 
# --- ROCK-COPYRIGHT-NOTE-END ---
#
# [MAIN] 20 rockplug ROCK-Plug/Drivers Configuration

######################################################################
# DEFAULT CONFIG GENERATION
######################################################################

gen_conf_sysinit() {
    [ -f /etc/conf/hardware ] && cp /etc/conf/hardware /etc/conf/hardware~
    cat <<EOF > /etc/conf/hardware
# Use ROCK-Plug to do hardware detection
HARDWARE_SETUP=rockplug

# The system init takes place before the root 
# filesystem is mounted read-write. If any module
# has to be loaded before this point, add it
# to the SYSINIT_MODULES variable
SYSINIT_MODULES="$SYSINIT_MODULES"
EOF
}

gen_conf_pci() {
    f=/etc/conf/pci
    [ -f $f ] && cp $f $f~
    cat <<EOF > $f
# PCI_SKIP_MODULES: Modules that should not be loaded 
# wether at system boot nor on a hotplug event.
PCI_SKIP_MODULES="$PCI_SKIP_MODULES"

# PCI_MODULES: Modules that should be loaded statically
# even if not detected or additionally to the hardware scan.
PCI_MODULES="$PCI_MODULES"

# USB host controller module override
PCI_USB_HOST=""
EOF
}

gen_conf_isapnp() {
    f=/etc/conf/isapnp
    [ -f $f ] && cp $f $f~
    cat <<EOF > $f
# ISAPNP_SKIP_MODULES: Modules that should not be loaded 
# wether at system boot nor on a hotplug event.
ISAPNP_SKIP_MODULES="$ISAPNP_SKIP_MODULES"

# ISAPNP_MODULES: Modules that should be loaded statically
# even if not detected or additionally to the hardware scan.
ISAPNP_MODULES="$ISAPNP_MODULES"
EOF
}

gen_conf_usb() {
    f=/etc/conf/usb
    [ -f $f ] && cp $f $f~
    cat <<EOF > $f
# Static list of USB modules
# These modules will be loaded on init.
# (for XFree add "mousedev input")
USB_MODULES=""

# Modules to be skipped
# These modules are never loaded, whether on init
# nor on a hotplug event.
USB_SKIP_MODULES=""
EOF
}

######################################################################
# HELPERS
######################################################################

var_append() {
    eval "[ \"\$$1\" ] && $1=\"\${$1}$2\"" || true
    eval "$1=\"\${$1}\$3\""
}
var_insert() {
    eval "[ \"\$$1\" ] && $1=\"$2\$$1\"" || true
    eval "$1=\"\$3\$$1\""
}
var_remove() {
    local a=${2//\/\\/}
    local b=${3//\/\\/}
    eval '[ "$'$1'" = "$3" ] && '$1'="" || true'
    eval $1'="${'$1'//$a$b$a/$2}"'
    eval $1'="${'$1'%$a$b}"'
    eval $1'="${'$1'#$b$a}"'
}

module_status() {
    # conf=$1 prefix=$2 module=$3
    local tmp

    [ -f /etc/conf/$1 ] && . /etc/conf/$1 

    eval "tmp=\"\$${2}_SKIP_MODULES\""
    var_remove tmp ' ' $3
    eval "[ \"$tmp\" != \"\$${2}_SKIP_MODULES\" ]" && return 2
    
    eval "tmp=\"\$${2}_MODULES\""
    var_remove tmp ' ' $3
    eval "[ \"$tmp\" != \"\$${2}_MODULES\" ]" && return 1

    return 0
}

module_cycle() {
    # conf=$1 prefix=$2 module=$3
    [ -f /etc/conf/$1 ] && . /etc/conf/$1 
    module_status $1 $2 $3
    case "$?" in
	0)
	    eval "var_append ${2}_MODULES ' ' $3"
	    eval "var_remove ${2}_SKIP_MODULES ' ' $3"
	    ;;
	1)
	    eval "var_remove ${2}_MODULES ' ' $3"
	    eval "var_append ${2}_SKIP_MODULES ' ' $3"
	    ;;
	2)
	    eval "var_remove ${2}_MODULES ' ' $3"
	    eval "var_remove ${2}_SKIP_MODULES ' ' $3"
	    ;;
    esac
    eval "gen_conf_$1"
}

set_var() {
    case $1 in 
	sysinit|pci|isapnp|usb)
	    gui_input "Set $2" "${!2}" "$2"
	    eval gen_conf_$1
	    ;;
    esac
}

edit() {
    gui_edit "Edit file $1" "$1"
    exec $STONE rockplug
}


######################################################################
# HELP MESSAGES
######################################################################

help() {
    case "$1" in
	sysinit)
	    gui_message "Sysinit configuration:

The system init modules (SYSINIT_MODULES)
are used in case modules have to be loaded
before the root file-system has been
mounted read-write."
	    ;;
	pci)
	    gui_message "PCI/Cardbus configuration:

Auto-detect devices: This gives you a list 
of the detected devices, including matched
modules ('auto' -> loaded if detected, 
  'skip' -> see PCI_SKIP_MODULES, 
  'load' -> see PCI_MODULES) 
and edit associated scripts.

PCI_SKIP_MODULES: Space seperated list of 
modules that should be skipped generally

PCI_MODULES: List of modules that should 
be loaded on system boot

PCI_USB_HOST: Set this variable to override
the automatic detection of the USB controller.
If you have a non-PCI USB controller, 
use USB_MODULES to load it (see USB setup).

View/Edit configuration: Do everything manually."
	    ;;
	isapnp)
	    gui_message "ISA/PNP configuration:

ISAPNP_SKIP_MODULES: Space seperated list of 
modules that should be skipped generally

ISAPNP_MODULES: List of modules that should 
be loaded on system boot

View/Edit configuration: Do everything manually."
	    ;;
	usb)
	    gui_message "USB configuration:

USB_SKIP_MODULES: Space seperated list of 
modules that should be skipped generally

USB_MODULES: List of modules that should 
be loaded on system boot

View/Edit configuration: Do everything manually."
	    ;;
	sound)
	    gui_message "Sound system selection:

You can choose between OSS, the Open Sound System
and ALSA, the Advanced Linux Sound Architecture."
	    ;;
    esac
}

######################################################################
# SUB-SYSTEM: SYSINIT
######################################################################

rp_conf_sysinit() {
    local cmd
    while
    [ -f /etc/conf/hardware ] && . /etc/conf/hardware
    cmd="gui_menu rockplug 'System init configuration.

Set system init modules
and/or edit configuration manually:'" 
    cmd="$cmd 'SYSINIT_MODULES: $SYSINIT_MODULES' 'set_var sysinit SYSINIT_MODULES'"
    cmd="$cmd '' ''"
    cmd="$cmd 'Help' 'help sysinit'"
    eval "$cmd"
      do : ; done
}

######################################################################
# SUB-SYSTEM: PCI/Cardbus
######################################################################
rp_conf_pci() {
    local cmd
    local each
    local detect
    while
    [ -f /etc/conf/pci ] || gen_conf_pci
    . /etc/conf/pci
    cmd="gui_menu rockplug 'PCI/Cardbus configuration.

Set options for auto-detectable modules/scripts
and/or edit configuration manually:'" 

    cmd="$cmd 'Auto-detect devices' 'rp_conf_pci_hwscan'"
    cmd="$cmd 'PCI_SKIP_MODULES: $PCI_SKIP_MODULES' 'set_var pci PCI_SKIP_MODULES'"
    cmd="$cmd 'PCI_MODULES: $PCI_MODULES' 'set_var pci PCI_MODULES'"
    cmd="$cmd 'PCI_USB_HOST: $PCI_USB_HOST' 'set_var pci PCI_USB_HOST'"
    cmd="$cmd 'View/Edit configuration file' 'edit /etc/conf/pci'"
    cmd="$cmd '' ''"
    cmd="$cmd 'Help' 'help pci'"

    eval "$cmd"
      do : ; done
}

rp_conf_pci_bus() {
    local cmd
    local bus=$1
    local busid="`echo $bus | tr ":." "__"`"

    while
#FIXME stone's line breaking mechanism does not cope with this:
    eval "cmd=\"gui_menu rockplug 'PCI/Cardbus device configuration \$bus\n\$class_D_$busid:\n  \$device_D_$busid\n  (\$vendor_D_$busid)\n  Prog-if: \$progif_D_$busid [\$subclass_D_$busid]\n  Subsystem: \$subdevice_D_$busid (\$subvendor_D_$busid)\n



'\""

    eval "modules=\"\$modules_$busid\""

    for each in $modules; do
	module_status pci PCI $each
	case "$?" in
	    0) status=auto ;;
	    1) status=load ;;
	    2) status=skip ;;
	esac
	cmd="$cmd '[$status] $each' 'module_cycle pci PCI $each'"
    done

    cmd="$cmd 'Add a script for device (not implemented yet)' 'rp_conf_pci_add_script $bus'"

    eval "$cmd"
      do : ; done
    
    return 0
}

rp_conf_pci_hwscan() {
    local cmd
    local each
    local detect
    while
    cmd="gui_menu rockplug 'PCI/Cardbus configuration.

This is a list of auto-detected PCI devices, 
select to configure/show more information:'" 

    echo "Detecting PCI devices..."
    if [ -z "$busses" ]; then
	while read bus device vendor_D device_D \
	    subdevice subvendor_D subdevice_D \
	    class class_D subclass_D progif_D \
	    modules scripts; do
	  echo "$bus $class_D: $device_D ($vendor_D)"
	  export busses="$busses $bus"
	  busid="`echo $bus | tr ":." "__"`"

	  for i in class class_D subclass_D progif_D \
	      device device_D vendor_D \
	      subdevice subdevice_D subvendor_D \
	      modules scripts; do
	    eval "export ${i}_$busid=\$$i"
	  done
	  
	  cmd="$cmd '$bus $class_D: $device_D ($vendor_D)' 'rp_conf_pci_bus $bus'"
	  detect="y"
	done < <(/sbin/rockscan pci)
    else
	for bus in $busses; do
	    busid="`echo $bus | tr ":." "__"`"
	    eval "cmd=\"\$cmd '\$bus \$class_D_$busid: \$device_D_$busid (\$vendor_D_$busid)' 'rp_conf_pci_bus $bus'\""
	    detect=y
	done
    fi
    [ -z "$detect" ] \
	&& cmd="$cmd '(No devices auto-detected.)' ''"
    eval "$cmd"
      do : ; done
}

######################################################################
# SUB-SYSTEM: ISA/PNP
######################################################################
rp_conf_isapnp() {
    local cmd
    local each
    local detect
    while
    [ -f /etc/conf/isapnp ] || gen_conf_isapnp
    . /etc/conf/isapnp
    cmd="gui_menu rockplug 'ISA/PNP configuration.

Set options for auto-detectable modules/scripts
and/or edit configuration manually:'" 

    cmd="$cmd 'Auto-detect devices (not yet implemented)' 'rp_conf_isapnp_hwscan'"
    cmd="$cmd 'ISAPNP_SKIP_MODULES: $ISAPNP_SKIP_MODULES' 'set_var isapnp ISAPNP_SKIP_MODULES'"
    cmd="$cmd 'ISAPNP_MODULES: $ISAPNP_MODULES' 'set_var pci ISAPNP_MODULES'"
    cmd="$cmd 'View/Edit configuration file' 'edit /etc/conf/isapnp'"
    cmd="$cmd '' ''"
    cmd="$cmd 'Help' 'help isapnp'"

    eval "$cmd"
      do : ; done
}

######################################################################
# SUB-SYSTEM: USB
######################################################################
rp_conf_usb() {
    local cmd
    local each
    local detect
    while
    [ -f /etc/conf/usb ] || gen_conf_usb
    . /etc/conf/usb
    cmd="gui_menu rockplug 'USB configuration.

Set options for auto-detectable modules/scripts
and/or edit configuration manually:'" 

    cmd="$cmd 'Auto-detect devices' 'rp_conf_usb_hwscan'"
    cmd="$cmd 'USB_SKIP_MODULES: $USB_SKIP_MODULES' 'set_var usb USB_SKIP_MODULES'"
    cmd="$cmd 'USB_MODULES: $USB_MODULES' 'set_var pci USB_MODULES'"
    cmd="$cmd 'View/Edit configuration file' 'edit /etc/conf/usb'"
    cmd="$cmd '' ''"
    cmd="$cmd 'Help' 'help usb'"

    eval "$cmd"
      do : ; done
}

rp_conf_usb_bus() {
    local cmd
    local device=$1
    local deviceid="`echo $device | tr "/" "_"`"

    while
    eval "cmd=\"gui_menu rockplug 'USB device configuration \$device\n\$product_$deviceid:  \$device_D_$deviceid\n  (\$vendor_D_$deviceid)\n
 Interface \$interface_$deviceid: \$interface_class_D_$deviceid (\$interface_subclass_D_$deviceid, \$interface_protocoll_D_$deviceid)\n
 Type \$device_type_$deviceid:      \$device_class_D_$deviceid (\$device_subclass_D_$deviceid, \$device_protocol_D_$deviceid)\n









'\""

    eval "modules=\"\$modules_$deviceid\""

    for each in $modules; do
	module_status usb USB $each
	case "$?" in
	    0) status=auto ;;
	    1) status=load ;;
	    2) status=skip ;;
	esac
	cmd="$cmd '[$status] $each' 'module_cycle usb USB $each'"
    done

    cmd="$cmd 'Add a script for device (not implemented yet)' 'rp_conf_usb_add_script $device'"

    eval "$cmd"
      do : ; done
    
    return 0
}

rp_conf_usb_hwscan() {
    local cmd
    local each
    local detect
    while
    cmd="gui_menu rockplug 'USB configuration.

This is a list of auto-detected USB devices, 
select to configure/show more information:'" 

    echo "Detecting USB devices..."
    if [ -z "$devices" ]; then
	while read device product \
	    vendor_D device_D \
	    interface interface_class_D interface_subclass_D interface_protocoll_D \
	    device_type device_class_D device_subclass_D device_protocol_D \
	    modules scripts; do
	  device="`echo $device | sed 's,/proc/bus/usb/,,'`"

	  echo "$device $class_D: $device_D ($vendor_D)"
	  export devices="$devices $device"
	  deviceid="`echo $device | tr "/" "_"`"

	  for i in product \
	      vendor_D device_D \
	      interface interface_class_D interface_subclass_D interface_protocoll_D \
	      device_type device_class_D device_subclass_D device_protocol_D \
	      modules scripts; do
	    eval "export ${i}_$deviceid=\$$i"
	  done
	  
	  cmd="$cmd '$device $product: $device_D ($vendor_D)' 'rp_conf_usb_bus $device'"
	  detect="y"
	done < <(/sbin/rockscan usb)
    else
	for device in $devices; do
	  deviceid="`echo $device | tr "/" "_"`"
	  eval "cmd=\"\$cmd '\$device \$product_$deviceid: \$device_D_$deviceid (\$vendor_D_$deviceid)' 'rp_conf_usb_bus $device'\""
	  detect=y
	done
    fi
    [ -z "$detect" ] \
	&& cmd="$cmd '(No devices auto-detected.)' ''"
    eval "$cmd"
      do : ; done
}

######################################################################
# SUB-SYSTEM: SCSI
######################################################################
rp_conf_scsi() {
    :
}

######################################################################
# SUB-SYSTEM: FireWire/IEEE1394
######################################################################
rp_conf_ieee1394() {
    :
}

######################################################################
# SUB-SYSTEM: MAC/IO
######################################################################
rp_conf_macio() {
    :
}

######################################################################
# SUB-SYSTEM: NET ???
######################################################################
rp_conf_net() {
    :
}

######################################################################
# SUB-SYSTEM: SOUND
######################################################################
rp_conf_sound() {
    local cmd
    while
    [ -f /etc/conf/sound ] && . /etc/conf/sound
    cmd="gui_menu rockplug 'Sound system selection.

Select the sound system (OSS/ALSA):'" 
    cmd="$cmd 'SOUND_SYSTEM: $SOUND_SYSTEM' 'set_var sound SOUND_SYSTEM'"
    cmd="$cmd '' ''"
    cmd="$cmd 'Help' 'help sound'"
    eval "$cmd"
      do : ; done
}

main() {
    local cmd
    while
    cmd="gui_menu rockplug 'Hardware/ROCK-Plug configuration

Select the sub-system you want to edit:'"
    cmd="$cmd 'Configure system init (rootfs read-only)' 'rp_conf_sysinit'"
    cmd="$cmd 'Configure PCI/Cardbus sub-system' 'rp_conf_pci'"
    cmd="$cmd 'Configure ISA/PNP sub-system' 'rp_conf_isapnp'"
    cmd="$cmd 'Configure USB sub-system' 'rp_conf_usb'"
    cmd="$cmd 'Select sound sub-system' 'rp_conf_sound'"

    eval "$cmd"
      do : ; done
}

# Local variables:
#   mode: shell-script
#   basic-offset: 2
# End:
